@@ -16,20 +16,20 @@ from typing import Any, Callable
 
 class Window(tk.Tk):
     def __init__(
-        self, name: str | None=None, size: str | None=None,
+        self, title: str | None=None, size: str | None=None,
         resizable: tuple[bool, bool] | None=None, **kwargs
     ):
         """
         Tkinter Tk wrapper, simplifies give name and size to the window, also
         manages the delete window protocol.
         Params:
-            name: name of the window.
+            title: title of the window.
             size: size of the window in format "width"x"height".
             resizable: enable resize width and/or height of the window.
         """
         super().__init__(**kwargs)
-        if name is not None:
-            self.title(name)
+        if title is not None:
+            self.title(title)
         if size is not None:
             self.geometry(size)
         if resizable is not None:
@@ -40,6 +40,16 @@ class Window(tk.Tk):
         """Destroys the window and quits python."""
         self.destroy()
         exit()
+    
+    def __setitem__(self, key: str, value: Any) -> None:
+        if key == 'title':
+            self.title(value)
+        elif key == 'size':
+            self.geometry(value)
+        elif key == 'resizable':
+            self.resizable(value)
+        else:
+            super().__setitem__(key, value)
 
 
 class CustomText(tk.Text):
@@ -83,6 +93,7 @@ class TextLineNumbers(tk.Canvas):
         """
         tk.Canvas.__init__(self, *args, **kwargs)
         self.textwidget = None
+        self.configs = {}
 
     def attach(self, text_widget: CustomText) -> None:
         """Updates the attached CustomText."""
@@ -98,24 +109,45 @@ class TextLineNumbers(tk.Canvas):
             if dline is None:
                 break
             y = dline[1]
-            linenum = str(i).split(".")[0]
-            self.create_text(2, y, anchor="nw", text=linenum)
+            prefix = self.configs.get('prefix', '')
+            configs = self.configs.copy()
+            if prefix:
+                configs.pop('prefix')
+            linenum = prefix + str(i).split(".")[0]
+            self.create_text(5, y, anchor="nw", text=linenum, **configs)
             i = self.textwidget.index("%s+1line" % i)
+    
+    def __setitem__(self, key, value):
+        if key == 'fg':
+            self.configs['fill'] = value
+            self.redraw()
+        elif key in ('font', 'prefix', 'justify'):
+            self.configs[key] = value
+            self.redraw()
+        else:
+            super().__setitem__(key, value)
 
 
 class TextArea(tk.Frame):
-    def __init__(self, *args, linecounter=True, width=80, height=40, **kwargs):
+    def __init__(
+        self, *args, linecounter: bool | None=True,
+        scrollbar: bool | None=True, width: int | None=80,
+        height: int | None=40, **kwargs
+    ):
         """
         Advanced TextArea inspired from tkinter, it allows to display a line
         counter.
         """
         tk.Frame.__init__(self, *args, **kwargs)
-        self.text = CustomText(self, width=width, height=height)
-        self.vsb = tk.Scrollbar(
-            self, orient="vertical", command=self.text.yview
+        self.text = CustomText(
+            self, width=width, height=height, wrap=tk.NONE, undo=True
         )
-        self.text.configure(yscrollcommand=self.vsb.set)
-        self.vsb.pack(side="right", fill="y")
+        if scrollbar is True:
+            self.vsb = tk.Scrollbar(
+                self, orient="vertical", command=self.text.yview
+            )
+            self.text.configure(yscrollcommand=self.vsb.set)
+            self.vsb.pack(side="right", fill="y")
         self.text.pack(side="right", fill="both", expand=True)
 
         if linecounter is True:
@@ -124,6 +156,27 @@ class TextArea(tk.Frame):
             self.linenumbers.pack(side="left", fill="y")
             self.text.bind("<<Change>>", self._on_change)
             self.text.bind("<Configure>", self._on_change)
+    
+    def __setitem__(self, key, value):
+        if key in ('bg', 'font'):
+            self.text[key] = value
+            if self.__dict__.get('linenumbers'):
+                self.linenumbers[key] = value
+        elif key.startswith('line') and self.__dict__.get('linenumbers'):
+            self.linenumbers[key.replace('line', '')] = value
+        elif key.startswith('scroll'):
+            print(key, value)
+            self.vsb[key.replace('scroll', '')] = value
+        elif key in ('prefix', ):
+            self.linenumbers[key] = value
+        else:
+            self.text[key] = value
+    
+    def bind(self, event, func):
+        self.text.bind(event, func)
+    
+    def search(self, *args, **kwargs):
+        self.text.search(*args, **kwargs)
 
     def _on_change(self, event: tk.Event) -> None:
         """Updates the line numbers."""
@@ -139,6 +192,15 @@ class TextArea(tk.Frame):
             end: index of end to clear.
         """
         self.text.delete(start, end)
+    
+    def tag_add(self, *args, **kwargs):
+        self.text.tag_add(*args, **kwargs)
+    
+    def tag_config(self, *args, **kwargs):
+        self.text.tag_config(*args, **kwargs)
+    
+    def insert(self, *args, **kwargs):
+        self.text.insert(*args, **kwargs)
 
     def write(
         self, text: str, file: io.TextIOWrapper | None=None,
@@ -206,6 +268,8 @@ class PhoneEntry(tk.Frame):
         self, master: tk.Widget,
         text: str | None=None, extensions: bool=True,
         button: str | None=None, command: Callable=lambda: None,
+        regex: str | None='\+[0-9]{1,3}[0-9]{10}',
+        valid_fg: str | None='green', invalid_fg: str | None='red',
         **kwargs
     ):
         """
@@ -217,16 +281,17 @@ class PhoneEntry(tk.Frame):
             button: optional button name.
             command: optional command for button.
         """
-        super().__init__(master)
+        super().__init__(master, **kwargs)
 
         self.extension = extensions
-        self.pattern = kwargs.get('regex', '.*')
+        self.pattern = kwargs.get('regex', regex)
         self.is_valid = False
+        self.valid_fg = valid_fg
+        self.invalid_fg = invalid_fg
 
         if text is not None:
-            tk.Label(
-                self, text=text
-            ).grid(row=0, column=0)
+            self.label = tk.Label(self, text=text)
+            self.label.grid(row=0, column=0)
         if extensions is True:
             self.extension_combobox = ttk.Combobox(
                 self, values=PHONE_EXTENSIONS, width=5, state='readonly'
@@ -237,8 +302,9 @@ class PhoneEntry(tk.Frame):
         self.number_entry.grid(row=0, column=2)
         self.number_entry.bind('<Key>', self.update_config)
         if button is not None:
-            tk.Button(self, text=button, command=command).grid(row=0, column=3)
-
+            self.button = tk.Button(self, text=button, command=command)
+            self.button.grid(row=0, column=3)
+    
     def get(self) -> None:
         if not self.extension:
             return self.number_entry.get()
@@ -251,9 +317,76 @@ class PhoneEntry(tk.Frame):
 
     def validate(self, *args) -> None:
         if self.is_valid is True:
-            self.number_entry['fg'] = '#00ff00'
+            self.number_entry['fg'] = self.valid_fg
         else:
-            self.number_entry['fg'] = '#ff0000'
+            self.number_entry['fg'] = self.invalid_fg
+    
+    def __setitem__(self, key, value):
+        if key == 'text':
+            if self.__dict__.get('text'):
+                self.label['text'] = value
+            else:
+                self.label = tk.Label(self, text=value)
+                self.label.grid(row=0, column=0)
+        elif key == 'command':
+            if self.__dict__.get('button'):
+                self.button['command'] = value
+        elif key == 'button':
+            if self.__dict__.get('button'):
+                self.button['text'] = value
+            else:
+                self.button = tk.Button(self, text=value)
+                self.button.grid(row=0, column=3)
+        elif key == 'regex':
+            self.pattern = value
+        elif key == 'valid_fg':
+            self.valid_fg = value
+        elif key == 'invalid_fg':
+            self.invalid_fg = value
+        else:
+            super().__setitem__(key, value)
+
+
+class TextSpan(tk.Frame):
+    def __init__(self, master: tk.Widget):
+        self.master = master
+        self.counter = 0
+    
+    def load(self):
+        for i, config in enumerate(self.values):
+            if self.mode == 'row':
+                if isinstance(self.type, str):
+                    widget = getattr(tk, self.type)
+                else:
+                    widget = self.type
+                all_config = self.__dict__.get('config', {}) | config
+                widget = widget(self.master, **all_config)
+                setattr(self, f'item{self.counter}', widget)
+                widget.grid(row=i, column=self.column, sticky='nw')
+            elif self.mode == 'column': ...
+            else: ...
+            self.counter += 1
+
+    def __setitem__(self, key, value):
+        setattr(self, key, value)
+        if all([self.__dict__.get(k) for k in ('type', 'mode', 'values')]):
+            self.load()
+
+
+class WindowMenu(tk.Menu):
+    def __init__(self, master: tk.Widget, menu_dict: dict, **kwargs):
+        super().__init__(master, **kwargs)
+        master.config(menu=self)
+        for menu, options in menu_dict.items():
+            new_menu = tk.Menu(self)
+            for name, option in options.items():
+                if name == 'separator':
+                    new_menu.add_separator(**option)
+                elif name in ('tearoff', ):
+                    new_menu[name] = option
+                else:
+                    new_menu.add_command(label=name, **option)
+            self.add_cascade(label=menu, menu=new_menu)
 
 
 def filetypes(
@@ -311,6 +444,8 @@ def style(
         from_:  module where to import the widget.
         widget: if there is a widget already created it can be also styled, if
                 provided then name, master and from_ are not needed.
+    Returns:
+        widget: the given widget with all aplied properties.
 
     """
     if alias is None:
@@ -321,222 +456,9 @@ def style(
     if widget is None:
         widget = getattr(from_, name)(master, **init_config)
     for key, value in all_config.items():
-        widget[key] = value
-    return widget
-
-
-def get_chunk(a: str, b: str, lines: list[str]) -> tuple[list[str], list[str]]:
-    """
-    Extract the lines between widgets based on the name.
-    Params:
-        a: first widget.
-        b: second widget.
-        lines: lines of the source file.
-    Returns:
-        the remaining lines to not repeat and the stripped lines between
-        widgets, also the indent level.
-    """
-    if b is None:
-        return [], [line.strip() for line in lines], a.count('    ')
-    start = lines.index(a)
-    end = lines.index(b, start+1)
-    return (
-        lines[end:], [line.strip() for line in lines[start:end]],
-        a.count('    ')
-    )
-
-
-def find_attribute(name: str, sources: list[Any]) -> Any:
-    """
-    Returns the attribute from the given sources.
-    Params:
-        name: name of the attribute to return.
-        sources: list of sources to search from.
-    """
-    for source in sources:
-        try:
-            return getattr(source, name)
-        except AttributeError:
-            continue
-    raise AttributeError('Attribute not found')
-
-
-def widget_info(chunk: list[str], indent: int) -> dict:
-    """
-    Gets a list of lines of a widget and returns a dictionary with its
-    properties.
-    Params:
-        chunk: list of strings of a widget.
-    """
-    widget = {}
-    widget['widget'] = chunk.pop(0)
-    widget['indent'] = indent
-
-    for item in chunk:
-        key, value = item.split(': ', maxsplit=1)
-        try:
-            widget[key] = eval(value)
-        except SyntaxError:
+        if key.startswith('.'):
+            func = getattr(widget, key[1:])
+            func(value)
+        else:
             widget[key] = value
     return widget
-
-
-def get_widgets(lines: list[str], names: list[str]) -> list[dict]:
-    """
-    Evaluate the lines from the source file, checks between each of them, gets
-    a chunk and then its info.
-    Params:
-        lines: lines from the source file.
-        names: names of the widgets.
-    """
-    widgets = []
-    for a, b in pylejandria.tools.pair(names + [None], 2):
-        lines, chunk, indent = get_chunk(a, b, lines)
-        widget = widget_info(chunk, indent)
-        widgets.append((widget, widget['id']))
-    widgets.append((None, None))
-    return widgets
-
-
-def place_widget(widget: Any, info: dict) -> None:
-    """
-    Manages the place method of the widget based on its info.
-    Params:
-        widget: widget to place.
-        info: info of the widget.
-    """
-    if info.get('.pack'):
-        widget.pack(**info.pop('.pack'))
-    elif info.get('.place'):
-        widget.place(**info.pop('.place'))
-    elif info.get('.grid'):
-        widget.grid(**info.pop('.grid'))
-
-
-def widget_args(info: dict):
-    """
-    Extracts main arguments from the info of widget.
-    Params:
-        info: dictionary of properties of a widget.
-    """
-    args = []
-    if info.get('parent'):
-        args.append(info.pop('parent'))
-    if info.get('init'):
-        args.append(info.pop('init'))
-    return args
-
-
-def get_widget(widgets: dict, widget_id: str) -> Any:
-    """
-    Returns a widget based on its id, if it doesnt exists then raises an error.
-    Params:
-        widgets: dictionary of widgets with id as key.
-        widget_id: id of the widget to return.
-    """
-    if widget := widgets.get(widget_id):
-        return widget
-    raise AttributeError('widget not found')
-
-
-def parse_value(
-    widget: Any, key: str, value: str, module: Any, widgets: dict
-) -> None:
-    """
-    Applies rules to the given value to style a widget.
-    Params:
-        widget: widget to apply properties.
-        key: name of the property to apply.
-        value: value of the property to edit.
-        functions: dictionary of functions in case they are needed.
-    """
-    if key in ('indent', 'id'):
-        return
-    if key.startswith('.'):
-        func = getattr(widget, key[1:])
-        func(value)
-        return
-    if re.match('\$.+', str(value)):
-        value = value.replace('$', '')
-        if args := re.search('\(.*\)', value):
-            args = args.group()
-            value = value.replace(args, '')
-        str_args = [] if args is None else args[1:-1].split(' | ')
-        args = []
-        for str_arg in str_args:
-            if str_arg == 'self':
-                args.append(widget)
-            elif str_arg == 'self.master':
-                args.append(widget.master)
-            elif str_arg.startswith('#'):
-                args.append(get_widget(widgets, str_arg[1:]))
-            elif re.match('self\[.+\]', str_arg):
-                args.append(re.search('\[.+\]', str_arg).group()[1:-1])
-            else:
-                args.append(eval(str_arg))
-
-        func = getattr(module, value)
-        widget[key] = lambda: func(*args)
-    else:
-        widget[key] = value
-
-
-def assign_parent(widget: Any, info1: dict, info2: dict) -> None:
-    """
-    Assigns parent to the widget based on the indentation.
-    Params:
-        widget: current widget, is used to get the parent or be the parent.
-        info1: dictionary of the current widget.
-        info2: dictionary of the next widget.
-    """
-    parent = widget
-    indent1 = info1.get('indent')
-    indent2 = info2.get('indent') if info2 else None
-    if indent2 is None:
-        return None
-    if indent1 == indent2:
-        parent = parent.master
-    elif indent1 > indent2:
-        for _ in range(indent1 - indent2 + 1):
-            parent = parent.master
-    return parent
-
-
-def load(filename: str, file: str) -> tk.Widget:
-    """
-    Loads a file with extension *.tk and builds all the widgets, the idea is
-    to have a setup more or less like QML, a cascade of widgets, is meant to
-    simple UI.
-    Params:
-        filename: path of the *.tk file.
-
-    Returns:
-        tk.Widget: the builded widget from the given file.
-    """
-    with open(filename, 'r') as f:
-        lines = f.read().split('\n')
-        widget_names = [line for line in lines if ':' not in line]
-
-    module = pylejandria.tools.get_module(file)
-    widgets = get_widgets(lines, widget_names)
-    window = None
-    built = {}
-
-    for i in range(len(widgets)-1):
-        info1, info2 = widgets[i][0], widgets[i + 1][0]
-        widget = find_attribute(info1.pop('widget'), (tk, pylejandria.gui))
-        widget_arguments = widget_args(info1)
-        widget = widget(*widget_arguments)
-        if id_ := info1.get('id'):
-            built[id_] = widget
-        if window is None:
-            window = widget
-        place_widget(widget, info1)
-
-        for key, value in info1.items():
-            parse_value(widget, key, value, module, built)
-
-        if parent := assign_parent(widget, info1, info2):
-            info2['parent'] = parent
-
-    return window
